unit ABCintrU;

// DON'T DELETE THIS COMMENT !!!

{ "AlPhAbEt" - ("ABC") - Interpreter }

{--------------------------------------------}
{ Unit:     ABCintrU.pas                     }
{ Version:  0.14                             }
{                                            }
{ Coder:    Raiden                           }
{ Producer: ShiKai                           }
{                                            }
{ I'm NOT Liable For Damages Of ANY Kind !!! }
{--------------------------------------------}

// DON'T DELETE THIS COMMENT !!!

interface

uses
  ABCtypeU;

{$I ABCcompI.inc}

{$IfDef UseStackOverflowProtection}
const
  CStackTest = '$=.';
{$EndIf UseStackOverflowProtection}

function InterpretLine(const AValue : AnsiString) : AnsiString;

procedure DeInitializeInterpreter;
procedure InitializeInterpreter(const AReadFileValue : TReadValueMethod;
                                const AReadScreenValue : TReadValueMethod;
                                const AWriteFileValue : TWriteValueMethod;
                                const AWriteScreenValue : TWriteValueMethod);

implementation

uses
  ABCvarU,
  ABCsysU,
  ABCsymU;

function InterpretLine(const AValue : AnsiString) : AnsiString;
  function ExecuteBlock(const AParameter : AnsiChar;
                        var AResult : AnsiString) : Boolean;
  var
    BlockArray : PAnsiStringArray;
    BoolBuffer : Boolean;
    Index      : LongInt;
  begin
    BlockArray := CatchBlockArray(AParameter);
    BoolBuffer := true;
    if (BlockArray <> nil) then
    begin
      for Index := Low(BlockArray^) to High(BlockArray^) do
      begin
        AResult := ABCInterpretLine(BlockArray^[Index]);

        BoolBuffer := (AResult = CError00Text);
        if not(BoolBuffer) then
          Break;
      end;
    end;

    Result := BoolBuffer;
  end;

  function ExecuteExIfOperation(const AParameterA : AnsiChar;
                                const AParameterB : AnsiChar;
                                const AOperator : AnsiChar;
                                const ALeftLine : AnsiString;
                                var ATextResult : AnsiString) : Boolean;
  begin
    ATextResult := ABCError(000,
                            CEmpty); // no error

    case AOperator of
      CExIfDiffersOp :
      begin
        if ABCDiffers(AParameterA,
                      AParameterB) then
          ATextResult := ABCInterpretLine(ALeftLine);

        Result := (ATextResult = CError00Text);
      end;

      CExIfEqualsOp :
      begin
        if ABCEquals(AParameterA,
                     AParameterB) then
          ATextResult := ABCInterpretLine(ALeftLine);

        Result := (ATextResult = CError00Text);
      end;
    else
      ATextResult := ABCError(004,
                              AParameterA + AOperator + AParameterB + #032 + ALeftLine); // unknown operator encountered

      Result := false;
    end;

{$IfDef UseStackOverflowProtection}
    if VTestStack then
      VTestAfter := ABCGetESP;
{$EndIf UseStackOverflowProtection}
  end;

{$IfDef UseLocalRegisters}
  procedure ExecuteExValidityOperation(const AOperator : AnsiChar);
  begin
    case AOperator of
      CExValidityGlobalOp :
      begin
        if (VLocalRegisters <> nil) then
        begin
          if (VLocalRegisters.Count >= VInstanceCounter) then
          begin
            if (VLocalRegisters.Items[Pred(VInstanceCounter)] <> nil) then
              TRegistersItem(VLocalRegisters.Items[Pred(VInstanceCounter)]).LocalValidity := false;
          end;
        end;
      end;

      CExValidityLocalOp :
      begin
        if (VLocalRegisters <> nil) then
        begin
          if (VLocalRegisters.Count >= VInstanceCounter) then
          begin
            if (VLocalRegisters.Items[Pred(VInstanceCounter)] <> nil) then
              TRegistersItem(VLocalRegisters.Items[Pred(VInstanceCounter)]).LocalValidity := true;
          end;
        end;
      end;
    end;
  end;
{$EndIf UseLocalRegisters}

  function ExecuteExWhileOperation(const AParameterA : AnsiChar;
                                   const AParameterB : AnsiChar;
                                   const AOperator : AnsiChar;
                                   const ALeftLine : AnsiString;
                                   var ATextResult : AnsiString) : Boolean;
  var
    BoolBuffer : Boolean;
  begin
    ATextResult := ABCError(000,
                            CEmpty); // no error

    case AOperator of
      CExWhileOp :
      begin
        BoolBuffer := true;
        while (ABCEquals(AParameterA,
                         AParameterB) and
               BoolBuffer) do
        begin
          ATextResult := ABCInterpretLine(ALeftLine);

          BoolBuffer := (ATextResult = CError00Text);
        end;

        Result := BoolBuffer
      end;

      CExWhileNotOp :
      begin
        BoolBuffer := true;
        while (ABCDiffers(AParameterA,
                          AParameterB) and
               BoolBuffer) do
        begin
          ATextResult := ABCInterpretLine(ALeftLine);

          BoolBuffer := (ATextResult = CError00Text);
        end;

        Result := BoolBuffer
      end;
    else
      ATextResult := ABCError(004,
                              AParameterA + AOperator + AParameterB + #032 + ALeftLine); // unknown operator encountered

      Result := false;
    end;
  end;

  procedure ExecuteOperation(const AParameterA : AnsiChar;
                             const AParameterB : AnsiChar;
                             const AOperator : AnsiChar);
    procedure ExecuteCommunication;
    var
      BoolBuffer : Boolean;
      CharBuffer : AnsiChar;
    begin
      case ABCValue(CEx0Reg) of
        C0Val : // file
        begin
          case ABCValue(CEx9Reg) of
            C0Val : // read from file
            begin
              if Assigned(VReadFileValue) then
              begin
                VReadFileValue(CharBuffer,
                               BoolBuffer);
                AnsiCharToParameters(CharBuffer);

                if BoolBuffer then
                  ABCSet(CExFlagReg,
                         C1Val)
                else
                  ABCSet(CExFlagReg,
                         C0Val);
              end;
            end;

            C1Val : // write to file
            begin
              if Assigned(VWriteFileValue) then
              begin
                VWriteFileValue(ParametersToAnsiChar,
                                BoolBuffer);

                if BoolBuffer then
                  ABCSet(CExFlagReg,
                         C1Val)
                else
                  ABCSet(CExFlagReg,
                         C0Val);
              end;
            end;
          else
            ABCSet(CExFlagReg,
                   C0Val);
          end;
        end;

        C1Val : // screen
        begin
          case ABCValue(CEx9Reg) of
            C0Val : // read from screen
            begin
              if Assigned(VReadScreenValue) then
              begin
                VReadScreenValue(CharBuffer,
                                 BoolBuffer);
                AnsiCharToParameters(CharBuffer);

                if BoolBuffer then
                  ABCSet(CExFlagReg,
                         C1Val)
                else
                  ABCSet(CExFlagReg,
                         C0Val);
              end;
            end;

            C1Val : // write to screen
            begin
              if Assigned(VWriteScreenValue) then
              begin
                VWriteScreenValue(ParametersToAnsiChar,
                                  BoolBuffer);

                if BoolBuffer then
                  ABCSet(CExFlagReg,
                         C1Val)
                else
                  ABCSet(CExFlagReg,
                         C0Val);
              end;
            end;
          else
            ABCSet(CExFlagReg,
                   C0Val);
          end;
        end;
      end;
    end;

  var
    BoolBuffer : Boolean;
  begin
    BoolBuffer := true;

    case AOperator of
      CAndOp :
      begin
        ABCAnd(AParameterA,
               AParameterB);
      end;

      CBecomesOp :
      begin
        ABCSet(AParameterA,
               AParameterB);
      end;

      CBecomesNotOp :
      begin
        ABCSetNot(AParameterA,
                  AParameterB);
      end;

      CCheckMemEmptyOp :
      begin
        if ABCCheckMemEmpty(AParameterA,
                            AParameterB) then
          ABCSet(CExFlagReg,
                 C1Val)
        else
          ABCSet(CExFlagReg,
                 C0Val);
      end;

      CCheckMemEntryOp :
      begin
        if ABCCheckMemEntry(AParameterA,
                            AParameterB) then
          ABCSet(CExFlagReg,
                 C1Val)
        else
          ABCSet(CExFlagReg,
                 C0Val);
      end;

      COrOp :
      begin
        ABCOr(AParameterA,
              AParameterB);
      end;

      CReadMemOp :
      begin
        if ABCReadMem(AParameterA,
                      AParameterB) then
          ABCSet(CExFlagReg,
                 C1Val)
        else
          ABCSet(CExFlagReg,
                 C0Val);
      end;

      CTouchMemFirstOp :
      begin
        if ABCTouchMemFirst(AParameterA,
                            AParameterB) then
          ABCSet(CExFlagReg,
                 C1Val)
        else
          ABCSet(CExFlagReg,
                 C0Val);
      end;

      CTouchMemLastOp :
      begin
        if ABCTouchMemLast(AParameterA,
                           AParameterB) then
          ABCSet(CExFlagReg,
                 C1Val)
        else
          ABCSet(CExFlagReg,
                 C0Val);
      end;

      CWriteMemOp :
      begin
        if ABCWriteMem(AParameterA,
                       AParameterB) then
          ABCSet(CExFlagReg,
                 C1Val)
        else
          ABCSet(CExFlagReg,
                 C0Val);
      end;

      CXOrOp :
      begin
        ABCXOr(AParameterA,
               AParameterB);
      end;
    else
      BoolBuffer := false;
    end;

    if (BoolBuffer and
        (AParameterA = CEx9Reg)) then
      ExecuteCommunication;
  end;

  function FindNextSymbol(const AValue : AnsiString;
                          const AStartIndex : LongInt;
                          const AIgnoreValue : AnsiChar) : LongInt;
  var
    Index : LongInt;
  begin
    Result := Length(AValue);

    for Index := AStartIndex to Length(AValue) do
    begin
      if (AValue[Index] = AIgnoreValue) then
      begin
        Result := Index;

        Break;
      end;
    end;
  end;

  function GetLeftLine(const AValue : AnsiString;
                       const AStartIndex : LongInt;
                       const AStopValue : AnsiChar) : AnsiString;
  var
    Count : LongInt;
    Index : LongInt;
  begin
    if (AStopValue = CEmpty) then
      Result := Copy(AValue,
                     AStartIndex,
                     Length(AValue) - Pred(AStartIndex))
    else
    begin
      SetLength(Result,
                Length(AValue) - Pred(AStartIndex));

      Count := 000;
      for Index := AStartIndex to Length(AValue) do
      begin
        if (AValue[Index] <> AStopValue) then
        begin
          Inc(Count);

          Result[Count] := AValue[Index];
        end
        else
          Break;
      end;

      if (Count < Length(Result)) then
        SetLength(Result,
                  Count);
    end;
  end;

  function GetNextSymbol(const AValue : AnsiString;
                         const AStartIndex : LongInt;
                         const AIgnoreValue : AnsiChar) : LongInt;
  var
    Index : LongInt;
  begin
    Result := Length(AValue);

    for Index := AStartIndex to Length(AValue) do
    begin
      if (AValue[Index] <> AIgnoreValue) then
      begin
        Result := Index;

        Break;
      end;
    end;
  end;

var
  BreakLoop     : Boolean;
  CurrentChar   : AnsiChar;
  DoNotNext     : Boolean;
  IgnoreIndex   : LongInt;
  Index         : LongInt;

{$IfDef UseStackOverflowProtection}
  QuitProcedure : Boolean;
{$EndIf UseStackOverflowProtection}

  TempResult    : AnsiString;
  UsedOperator  : AnsiChar;
  UsedRegister  : AnsiChar;
begin
  Inc(VInstanceCounter);

  Result := ABCError(000,
                     CEmpty); // no error

{$IfDef UseStackOverflowProtection}
  QuitProcedure := false;

  if not(VTestStack) then
  begin
    if ((VInstanceCounter * VInterpreterSize) >= CMaxStackSize) then
    begin
      Result := ABCError(014,
                         AValue); // possible stack-overflow

      QuitProcedure := true;
    end;
  end;

  if not(QuitProcedure) then
  begin
{$EndIf UseStackOverflowProtection}

    if (Length(AValue) > 000) then
    begin
{$IfDef UseLocalRegisters}
      VLocalRegisters.InsertAsLast(TRegistersItem.Create);
{$EndIf UseLocalRegisters}

      BreakLoop    := false;
      CurrentChar  := CEmpty;
      DoNotNext    := false;
      IgnoreIndex  := 000;
      UsedOperator := CEmpty;
      UsedRegister := CEmpty;

      if VWaitForBlockEnd then
      begin
        if (VWaitForBlockArray <> CEmpty) then
          ABCAddBlockLine(VWaitForBlockArray,
                          '')
        else
        begin
          Result := ABCError(013,
                             AValue); // missing block-name

          BreakLoop := true;
        end;
      end;

      for Index := 001 to Length(AValue) do
      begin
        if (Index > IgnoreIndex) then
        begin
          CurrentChar := AValue[Index];

{$IfDef DebugMode}
          if (CurrentChar = CDebugModeChar) then
          begin
            DoDebugMode;

            DoNotNext := true;
          end;
{$EndIf DebugMode}

          if not(DoNotNext) then
          begin
            if (CurrentChar = CComment) then
              BreakLoop := true
            else
            begin
              if not(VWaitForBlockEnd) then
              begin
                if (CurrentChar <> CIgnored) then
                begin
                  if IsAcceptedSymbol(CurrentChar) then
                  begin

{$IfDef UseLocalRegisters}
                    if IsExValidityOperator(CurrentChar) then
                    begin
                      if ((UsedOperator = CEmpty) and
                          (UsedRegister = CEmpty)) then
                        ExecuteExValidityOperation(CurrentChar)
                      else
                      begin
                        Result := ABCError(015,
                                           AValue); // validity-switch not allowed within an operation

                        BreakLoop := true;
                      end;
                    end
                    else
                    begin
{$EndIf UseLocalRegisters}

                      if (IsOperator(CurrentChar) or
                          IsExIfOperator(CurrentChar) or
                          IsExWhileOperator(CurrentChar)) then
                      begin
                        if (UsedRegister <> CEmpty) then
                        begin
                          if (UsedOperator = CEmpty) then
                            UsedOperator := CurrentChar
                          else
                          begin
                            Result := ABCError(002,
                                               AValue); // operator already defined

                            BreakLoop := true;
                          end;
                        end
                        else
                        begin
                          Result := ABCError(003,
                                             AValue); // no first parameter defined

                          BreakLoop := true;
                        end;
                      end
                      else
                      begin
                        if IsRegister(CurrentChar) then
                        begin
                          if (UsedRegister = CEmpty) then
                            UsedRegister := CurrentChar
                          else
                          begin
                            if (UsedRegister <> CEmpty) then
                            begin
                              if (UsedOperator <> CEmpty) then
                              begin
                                if IsOperator(UsedOperator) then
                                  ExecuteOperation(UsedRegister,
                                                   CurrentChar,
                                                   UsedOperator)
                                else
                                begin
                                  if IsExIfOperator(UsedOperator) then
                                  begin
                                    BreakLoop := not(ExecuteExIfOperation(UsedRegister,
                                                                          CurrentChar,
                                                                          UsedOperator,
                                                                          GetLeftLine(AValue,
                                                                                      Succ(Index),
                                                                                      CEndIf),
                                                                          Result));

                                    if not(BreakLoop) then
                                    begin
                                      IgnoreIndex := FindNextSymbol(AValue,
                                                                    Succ(Index),
                                                                    CEndIf);

                                      BreakLoop := (IgnoreIndex > Length(AValue));
                                    end;
                                  end
                                  else
                                  begin
                                    if IsExWhileOperator(UsedOperator) then
                                    begin
                                      BreakLoop := not(ExecuteExWhileOperation(UsedRegister,
                                                                               CurrentChar,
                                                                               UsedOperator,
                                                                               GetLeftLine(AValue,
                                                                                           Succ(Index),
                                                                                           CEndWhile),
                                                                               Result));

                                      if not(BreakLoop) then
                                      begin
                                        IgnoreIndex := FindNextSymbol(AValue,
                                                                      Succ(Index),
                                                                      CEndWhile);

                                        BreakLoop := (IgnoreIndex > Length(AValue));
                                      end;
                                    end
                                    else
                                    begin
                                      Result := ABCError(004,
                                                         AValue); // unknown operator encountered

                                      BreakLoop := true;
                                    end;
                                  end;
                                end;

                                UsedOperator := CEmpty;
                                UsedRegister := CEmpty;
                              end
                              else
                              begin
                                Result := ABCError(005,
                                                   AValue); // no operator defined

                                BreakLoop := true;
                              end;
                            end
                            else
                            begin
                              Result := ABCError(003,
                                                 AValue); // no first parameter defined

                              BreakLoop := true;
                            end;
                          end;
                        end
                        else
                        begin
                          if (IsValue(CurrentChar) or
                              IsExValue(CurrentChar)) then
                          begin
                            if (UsedRegister <> CEmpty) then
                            begin
                              if (UsedOperator <> CEmpty) then
                              begin
                                if IsOperator(UsedOperator) then
                                  ExecuteOperation(UsedRegister,
                                                   CurrentChar,
                                                   UsedOperator)
                                else
                                begin
                                  if IsExIfOperator(UsedOperator) then
                                  begin
                                    BreakLoop := not(ExecuteExIfOperation(UsedRegister,
                                                                          CurrentChar,
                                                                          UsedOperator,
                                                                          GetLeftLine(AValue,
                                                                                      Succ(Index),
                                                                                      CEndIf),
                                                                          Result));

                                    if not(BreakLoop) then
                                    begin
                                      IgnoreIndex := FindNextSymbol(AValue,
                                                                    Succ(Index),
                                                                    CEndIf);

                                      BreakLoop := (IgnoreIndex > Length(AValue));
                                    end;
                                  end
                                  else
                                  begin
                                    if IsExWhileOperator(UsedOperator) then
                                    begin
                                      BreakLoop := not(ExecuteExWhileOperation(UsedRegister,
                                                                               CurrentChar,
                                                                               UsedOperator,
                                                                               GetLeftLine(AValue,
                                                                                           Succ(Index),
                                                                                           CEndWhile),
                                                                               Result));

                                      if not(BreakLoop) then
                                      begin
                                        IgnoreIndex := FindNextSymbol(AValue,
                                                                      Succ(Index),
                                                                      CEndWhile);

                                        BreakLoop := (IgnoreIndex > Length(AValue));
                                      end;
                                    end
                                    else
                                    begin
                                      Result := ABCError(004,
                                                         AValue); // unknown operator encountered

                                      BreakLoop := true;
                                    end;
                                  end;
                                end;

                                UsedOperator := CEmpty;
                                UsedRegister := CEmpty;
                              end
                              else
                              begin
                                Result := ABCError(005,
                                                   AValue); // no operator defined

                                BreakLoop := true;
                              end;
                            end
                            else
                            begin
                              Result := ABCError(003,
                                                 AValue); // no first parameter defined

                              BreakLoop := true;
                            end;
                          end
                          else
                          begin
                            Result := ABCError(001,
                                               AValue); // unknown symbol encountered

                            BreakLoop := true;
                          end;
                        end;
                      end;

{$IfDef UseLocalRegisters}
                    end;
{$EndIf UseLocalRegisters}

                  end
                  else
                  begin
                    case CurrentChar of
                      CBeginBlock :
                      begin
                        if not(VWaitForBlockEnd) then
                        begin
                          if (UsedRegister <> CEmpty) then
                          begin
                            if (UsedOperator = CEmpty) then
                            begin
                              if not(Pos(UsedRegister, VCommandBlockExecution) > 000) then
                              begin
                                VWaitForBlockArray := UsedRegister;
                                VWaitForBlockEnd   := true;

                                ABCClearBlockArray(VWaitForBlockArray);

                                UsedRegister := CEmpty;
                              end
                              else
                              begin
                                Result := ABCError(016,
                                                   AValue); // no self-defining

                                BreakLoop := true;
                              end;
                            end
                            else
                            begin
                              Result := ABCError(010,
                                                 AValue); // no command-block-beginning within an operation

                              BreakLoop := true;
                            end;
                          end
                          else
                          begin
                            Result := ABCError(003,
                                               AValue); // no first parameter defined

                            BreakLoop := true;
                          end;
                        end
                        else
                        begin
                          Result := ABCError(009,
                                             AValue); // no interlaced command-blocks

                          BreakLoop := true;
                        end;
                      end;

                      CEndBlock :
                      begin
                        Result := ABCError(011,
                                           AValue); // missing command-block-beginning

                        BreakLoop := true;
                      end;

                      CEndIf :
                      begin
                        Result := ABCError(017,
                                           AValue); // EndIf not withing an If-Statement

                        BreakLoop := true;
                      end;

                      CEndWhile :
                      begin
                        Result := ABCError(020,
                                           AValue); // EndWhile not withing a While-Statement

                        BreakLoop := true;
                      end;

                      CUseBlock :
                      begin
                        if (UsedRegister <> CEmpty) then
                        begin
                          if (UsedOperator = CEmpty) then
                          begin
                            VCommandBlockExecution := VCommandBlockExecution + UsedRegister;

                            if not(ExecuteBlock(UsedRegister,
                                                TempResult)) then
                            begin
                              Result := TempResult;

                              BreakLoop := true;
                            end
                            else
                              UsedRegister := CEmpty;

                            VCommandBlockExecution := Copy(VCommandBlockExecution,
                                                           001,
                                                           Pred(Length(VCommandBlockExecution)));
                          end
                          else
                          begin
                            Result := ABCError(008,
                                               AValue); // command-block within an operation

                            BreakLoop := true;
                          end;
                        end
                        else
                        begin
                          Result := ABCError(003,
                                             AValue); // no first parameter defined

                          BreakLoop := true;
                        end;
                      end;
                    else
                      Result := ABCError(001,
                                         AValue); // unknown symbol encountered

                      BreakLoop := true;
                    end;
                  end;
                end;
              end
              else
              begin
                if (VWaitForBlockArray <> CEmpty) then
                begin
                  if ((CurrentChar = CEndBlock) and
                      (Length(AValue) > Index) and
                      (AValue[GetNextSymbol(AValue, Succ(Index), CIgnored)] = VWaitForBlockArray)) then
                  begin
                    if VWaitForBlockEnd then
                    begin
                      if ((UsedRegister = CEmpty) and
                          (UsedOperator = CEmpty)) then
                      begin
                        VWaitForBlockArray := CEmpty;
                        VWaitForBlockEnd   := false;

                        DoNotNext := true;
                      end
                      else
                      begin
                        Result := ABCError(012,
                                           AValue); // no command-block-ending within an operation

                        BreakLoop := true;
                      end;
                    end
                    else
                    begin
                      Result := ABCError(011,
                                         AValue); // missing command-block-beginning

                      BreakLoop := true;
                    end;
                  end
                  else
                    ABCAddBlockChar(VWaitForBlockArray,
                                    CurrentChar);
                end
                else
                begin
                  Result := ABCError(013,
                                     AValue); // missing block-name

                  BreakLoop := true;
                end;
              end;
            end;
          end
          else
          begin
            if (CurrentChar <> CIgnored) then
              DoNotNext := false;
          end;

          if BreakLoop then
            Break;
        end;
      end;

      if not(BreakLoop) then
      begin
        if (UsedRegister <> CEmpty) then
        begin
          if (UsedOperator <> CEmpty) then
            Result := ABCError(006,
                               AValue) // no second parameter defined
          else
            Result := ABCError(007,
                               AValue); // no second parameter and operator defined
        end;
      end;

{$IfDef UseLocalRegisters}
      TRegistersItem(VLocalRegisters.Items[Pred(VLocalRegisters.Count)]).Free;
      VLocalRegisters.Items[Pred(VLocalRegisters.Count)] := nil;
      VLocalRegisters.DeleteLast;
{$EndIf UseLocalRegisters}
    end;

{$IfDef UseStackOverflowProtection}
  end;
{$EndIf UseStackOverflowProtection}

  Dec(VInstanceCounter);
end;

procedure DeInitializeInterpreter;
begin
  InitializeBlockArrays;
  DeInitializeMemoryList;

{$IfDef UseLocalRegisters}
  DeInitializeRegisters;
{$EndIf UseLocalRegisters}

  VReadFileValue    := nil;
  VReadScreenValue  := nil;
  VWriteFileValue   := nil;
  VWriteScreenValue := nil;
end;

procedure InitializeInterpreter(const AReadFileValue : TReadValueMethod;
                                const AReadScreenValue : TReadValueMethod;
                                const AWriteFileValue : TWriteValueMethod;
                                const AWriteScreenValue : TWriteValueMethod);
begin
  Randomize;

  InitializeBlockArrays;
  InitializeMemoryList;
  InitializeRegisters;

  VInstanceCounter := 000;

  VReadFileValue    := AReadFileValue;
  VReadScreenValue  := AReadScreenValue;
  VWriteFileValue   := AWriteFileValue;
  VWriteScreenValue := AWriteScreenValue;

{$IfDef UseStackOverflowProtection}
  VTestBefore := ABCGetESP;

  VTestStack := true;
  ABCInterpretLine(CStackTest);
  VTestStack := false;

  VInterpreterSize := VTestBefore - VTestAfter;
{$EndIf UseStackOverflowProtection}
end;

end.
