unit ABCsysU;

// DON'T DELETE THIS COMMENT !!!

{ "AlPhAbEt" - ("ABC") - Interpreter }

{--------------------------------------------}
{ Unit:     ABCsysU.pas                      }
{ Version:  0.14                             }
{                                            }
{ Coder:    Raiden                           }
{ Producer: ShiKai                           }
{                                            }
{ I'm NOT Liable For Damages Of ANY Kind !!! }
{--------------------------------------------}

// DON'T DELETE THIS COMMENT !!!

interface

{$I ABCcompI.inc}

procedure ABCAddBlockChar(const AValueA : AnsiChar;
                          const AValueB : AnsiChar);
procedure ABCAddBlockLine(const AValueA : AnsiChar;
                          const AValueB : AnsiString);
procedure ABCClearBlockArray(const AValue : AnsiChar);

procedure ABCAnd(const AValueA : AnsiChar;
                 const AValueB : AnsiChar);
procedure ABCOr(const AValueA : AnsiChar;
                const AValueB : AnsiChar);
procedure ABCXor(const AValueA : AnsiChar;
                 const AValueB : AnsiChar);

function ABCCheckMemEmpty(const AValueA : AnsiChar;
                          const AValueB : AnsiChar) : Boolean;
function ABCCheckMemEntry(const AValueA : AnsiChar;
                          const AValueB : AnsiChar) : Boolean;

function ABCDiffers(const AValueA : AnsiChar;
                    const AValueB : AnsiChar) : Boolean;
function ABCEquals(const AValueA : AnsiChar;
                   const AValueB : AnsiChar) : Boolean;

function ABCError(const AValue : LongInt;
                  const AParameter : AnsiString) : AnsiString;

{$IfDef UseStackOverflowProtection}
function ABCGetESP : LongWord;
{$EndIf UseStackOverflowProtection}

function ABCGetRegister(const AValue : AnsiChar) : PAnsiChar;
procedure ABCSetRegister(const AValueA : AnsiChar;
                         const AValueB : AnsiChar); overload;
procedure ABCSetRegister(const AValueA : PAnsiChar;
                         const AValueB : AnsiChar); overload;

function ABCInterpretLine(const AValue : AnsiString) : AnsiString;

function ABCRandom : AnsiChar;

function ABCReadMem(const AValueA : AnsiChar;
                    const AValueB : AnsiChar) : Boolean;
function ABCWriteMem(const AValueA : AnsiChar;
                     const AValueB : AnsiChar) : Boolean;

procedure ABCSet(const AValueA : AnsiChar;
                 const AValueB : AnsiChar);
procedure ABCSetNot(const AValueA : AnsiChar;
                    const AValueB : AnsiChar);

function ABCTouchMemFirst(const AValueA : AnsiChar;
                          const AValueB : AnsiChar) : Boolean;
function ABCTouchMemLast(const AValueA : AnsiChar;
                         const AValueB : AnsiChar) : Boolean;

function ABCValue(const AValue : AnsiChar) : AnsiChar;

const
  CError00Text = '#000 No Error Encountered';
  CError01Text = '#001 Unknown Symbol Encountered:';
  CError02Text = '#002 Operator Already Defined:';
  CError03Text = '#003 No First Parameter Defined:';
  CError04Text = '#004 Unknown Operator Encountered:';
  CError05Text = '#005 No Operator Defined:';
  CError06Text = '#006 No Second Parameter Defined:';
  CError07Text = '#007 No Second Parameter And Operator Defined:';
  CError08Text = '#008 Block Cannot Be Executed Within An Operation:';
  CError09Text = '#009 No Interlaced Blocks Allowed:';
  CError10Text = '#010 No Block-Beginning Within An Operation Allowed:';
  CError11Text = '#011 Missing Block-Beginning:';
  CError12Text = '#012 No Block-Ending Within An Operation Allowed:';
  CError13Text = '#013 Missing Block-Name:';

{$IfDef UseStackOverflowProtection}
  CError14Text = '#014 Possible Stack-Overflow Encountered:';
{$EndIf UseStackOverflowProtection}

{$IfDef UseLocalRegisters}
  CError15Text = '#015 No Validity-Switch Allowed Within An Operation:';
{$EndIf UseLocalRegisters}

  CError16Text = '#016 No Definition Of A Block Within Itself Allowed:';
  CError17Text = '#017 An EndIf-Symbol Can Only Be Used Within An If-Statement:';

{$IfDef UseLocalRegisters}
  CError18Text = '#018 Recursion Could Not Be Executed:';
{$EndIf UseLocalRegisters}

  CError19Text = '#019 Unknown Second Parameter Encountered:';
  CError20Text = '#020 An EndWhile-Symbol Can Only Be Used Within A While-Statement:';

implementation

uses
  ABCvarU,
  ABCtypeU,
  ABCsymU,
  ABCintrU;

procedure ABCAddBlockChar(const AValueA : AnsiChar;
                          const AValueB : AnsiChar);
var
  BlockArray : PAnsiStringArray;
begin
  BlockArray := CatchBlockArray(AValueA);

  if (BlockArray <> nil) then
  begin
    if (Length(BlockArray^) <= 000) then
      SetLength(BlockArray^,
                001);

    BlockArray^[High(BlockArray^)] := BlockArray^[High(BlockArray^)] + AValueB;
  end;
end;

procedure ABCAddBlockLine(const AValueA : AnsiChar;
                          const AValueB : AnsiString);
var
  BlockArray : PAnsiStringArray;
begin
  BlockArray := CatchBlockArray(AValueA);

  if (BlockArray <> nil) then
  begin
    if (Length(BlockArray^) <= 000) then
      SetLength(BlockArray^,
                001)
    else
      SetLength(BlockArray^,
                Succ(Length(BlockArray^)));

    BlockArray^[High(BlockArray^)] := AValueB;
  end;
end;

procedure ABCClearBlockArray(const AValue : AnsiChar);
var
  BlockArray : PAnsiStringArray;
begin
  BlockArray := CatchBlockArray(AValue);

  if (BlockArray <> nil) then
    SetLength(BlockArray^,
              000);
end;

procedure ABCAnd(const AValueA : AnsiChar;
                 const AValueB : AnsiChar);
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  ValRegister := ABCGetRegister(AValueA);
  ValValue    := ABCValue(AValueB);

  if ((ValRegister <> nil) and
      (ValValue <> CEmpty)) then
  begin
    if ((ValRegister^ = C1Val) and
        (ValValue = C1Val)) then
      ValRegister^ := C1Val
    else
      ValRegister^ := C0Val;
  end;
end;

procedure ABCOr(const AValueA : AnsiChar;
                const AValueB : AnsiChar);
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  ValRegister := ABCGetRegister(AValueA);
  ValValue    := ABCValue(AValueB);

  if ((ValRegister <> nil) and
      (ValValue <> CEmpty)) then
  begin
    if ((ValRegister^ = C1Val) or
        (ValValue = C1Val)) then
      ValRegister^ := C1Val
    else
      ValRegister^ := C0Val;
  end;
end;

procedure ABCXor(const AValueA : AnsiChar;
                 const AValueB : AnsiChar);
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  ValRegister := ABCGetRegister(AValueA);
  ValValue    := ABCValue(AValueB);

  if ((ValRegister <> nil) and
      (ValValue <> CEmpty)) then
  begin
    if (((ValRegister^ = C0Val) and
         (ValValue = C1Val)) or
        ((ValRegister^ = C1Val) and
         (ValValue = C0Val))) then
      ValRegister^ := C1Val
    else
      ValRegister^ := C0Val;
  end;
end;

function ABCCheckMemEmpty(const AValueA : AnsiChar;
                          const AValueB : AnsiChar) : Boolean;
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  Result := false;

  if (VMemoryList <> nil) then
  begin
    ValRegister := ABCGetRegister(AValueA);
    ValValue    := ABCValue(AValueB);

    if (ValRegister <> nil) and
       (ValValue <> CEmpty) then
    begin
      if (ValValue = C0Val) then
      begin
        if (VMemoryList.Count <= 000) then
          ValRegister^ := C0Val
        else
          ValRegister^ := C1Val;

        Result := true;
      end
      else
      begin
        if (VMemoryList.Count <= 000) then
          ValRegister^ := C1Val
        else
          ValRegister^ := C0Val;

        Result := true;
      end;
    end;
  end;
end;

function ABCCheckMemEntry(const AValueA : AnsiChar;
                          const AValueB : AnsiChar) : Boolean;
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  Result := false;

  if (VMemoryList <> nil) then
  begin
    ValRegister := ABCGetRegister(AValueA);
    ValValue    := ABCValue(AValueB);

    if (ValRegister <> nil) and
       (ValValue <> CEmpty) then
    begin
      if (ValValue = C0Val) then
      begin
        if (VMemoryList.Count > 000) then
        begin
          if VMemoryList.IsFirstAddedEntry(000) then
            ValRegister^ := C1Val
          else
            ValRegister^ := C0Val;
        end
        else
          ValRegister^ := C1Val;

        Result := true;
      end
      else
      begin
        if (VMemoryList.Count > 000) then
        begin
          if VMemoryList.IsFirstAddedEntry(Pred(VMemoryList.Count)) then
            ValRegister^ := C1Val
          else
            ValRegister^ := C0Val;
        end
        else
          ValRegister^ := C1Val;

        Result := true;
      end;
    end;
  end;
end;

function ABCDiffers(const AValueA : AnsiChar;
                    const AValueB : AnsiChar) : Boolean;
var
  ValValueA : AnsiChar;
  ValValueB : AnsiChar;
begin
  Result := false;

  ValValueA := ABCValue(AValueA);
  ValValueB := ABCValue(AValueB);

  if ((ValValueA <> CEmpty) and
      (ValValueB <> CEmpty)) then
    Result := (((ValValueA = C0Val) and
                (ValValueB = C1Val)) or
               ((ValValueA = C1Val) and
                (ValValueB = C0Val)));
end;

function ABCEquals(const AValueA : AnsiChar;
                   const AValueB : AnsiChar) : Boolean;
var
  ValValueA : AnsiChar;
  ValValueB : AnsiChar;
begin
  Result := false;

  ValValueA := ABCValue(AValueA);
  ValValueB := ABCValue(AValueB);

  if ((ValValueA <> CEmpty) and
      (ValValueB <> CEmpty)) then
    Result := (((ValValueA = C0Val) and
                (ValValueB = C0Val)) or
               ((ValValueA = C1Val) and
                (ValValueB = C1Val)));
end;

function ABCError(const AValue : LongInt;
                  const AParameter : AnsiString) : AnsiString;
begin
  case AValue of
    000 : Result := CError00Text;
    001 : Result := CError01Text + #032 + AParameter;
    002 : Result := CError02Text + #032 + AParameter;
    003 : Result := CError03Text + #032 + AParameter;
    004 : Result := CError04Text + #032 + AParameter;
    005 : Result := CError05Text + #032 + AParameter;
    006 : Result := CError06Text + #032 + AParameter;
    007 : Result := CError07Text + #032 + AParameter;
    008 : Result := CError08Text + #032 + AParameter;
    009 : Result := CError09Text + #032 + AParameter;
    010 : Result := CError10Text + #032 + AParameter;
    011 : Result := CError11Text + #032 + AParameter;
    012 : Result := CError12Text + #032 + AParameter;
    013 : Result := CError13Text + #032 + AParameter;

{$IfDef UseStackOverflowProtection}
    014 : Result := CError14Text + #032 + AParameter;
{$EndIf UseStackOverflowProtection}

{$IfDef UseLocalRegisters}
    015 : Result := CError15Text + #032 + AParameter;
{$EndIf UseLocalRegisters}

    016 : Result := CError16Text + #032 + AParameter;
    017 : Result := CError17Text + #032 + AParameter;

{$IfDef UseLocalRegisters}
    018 : Result := CError18Text + #032 + AParameter;
{$EndIf UseLocalRegisters}

    019 : Result := CError19Text + #032 + AParameter;
    020 : Result := CError20Text + #032 + AParameter;
  else
    Result := 'UNKNOWN ERROR'
  end;
end;

{$IfDef UseStackOverflowProtection}
function ABCGetESP : LongWord;
asm
  MOV Result, ESP
end;
{$EndIf UseStackOverflowProtection}

function ABCGetRegister(const AValue : AnsiChar) : PAnsiChar;
begin
  Result := nil;

{$IfDef UseLocalRegisters}
  if (VLocalRegisters <> nil) then
  begin
    if (VLocalRegisters.Count >= VInstanceCounter) then
    begin
      if (VLocalRegisters.Items[Pred(VInstanceCounter)] <> nil) then
      begin
        if TRegistersItem(VLocalRegisters.Items[Pred(VInstanceCounter)]).LocalValidity then
          Result := TRegistersItem(VLocalRegisters.Items[Pred(VInstanceCounter)]).CatchRegister(AValue)
        else
{$EndIf UseLocalRegisters}

          Result := CatchRegister(AValue);

{$IfDef UseLocalRegisters}
      end;
    end;
  end;
{$EndIf UseLocalRegisters}
end;

procedure ABCSetRegister(const AValueA : AnsiChar;
                         const AValueB : AnsiChar);
begin
  ABCSetRegister(ABCGetRegister(AValueA),
                 AValueB);
end;

procedure ABCSetRegister(const AValueA : PAnsiChar;
                         const AValueB : AnsiChar);
var
  ValValue : AnsiChar;
begin
  if (AValueA <> nil) then
  begin
    ValValue := ABCValue(AValueB);

    if (ValValue <> CEmpty) then
      AValueA^ := ValValue;
  end;
end;

function ABCInterpretLine(const AValue : AnsiString) : AnsiString;
begin
{$IfDef UseLocalRegisters}
  Result := ABCError(018,
                     AValue); // recursion not executable

  if (VLocalRegisters <> nil) then
  begin
    if (VLocalRegisters.Count >= Succ(VInstanceCounter)) then
    begin
      if (VLocalRegisters.Items[VInstanceCounter] <> nil) then
      begin
        TRegistersItem(VLocalRegisters.Items[VInstanceCounter]).Initialize;
{$EndIf UseLocalRegisters}

        Result := InterpretLine(AValue);

{$IfDef UseLocalRegisters}
      end;
    end;
  end;
{$EndIf UseLocalRegisters}
end;

function ABCRandom : AnsiChar;
begin
  Result := ByteToABC(Random(002));
end;

function ABCReadMem(const AValueA : AnsiChar;
                    const AValueB : AnsiChar) : Boolean;
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  Result := false;

  if (VMemoryList <> nil) then
  begin
    if (VMemoryList.Count > 000) then
    begin
      ValRegister := ABCGetRegister(AValueA);
      ValValue    := ABCValue(AValueB);

      if (ValRegister <> nil) and
         (ValValue <> CEmpty) then
      begin
        if (ValValue = C0Val) then
        begin
          ValRegister^ := VMemoryList.Items[000];

          if (ValRegister^ = VMemoryList.Items[000]) then
            Result := VMemoryList.DeleteFirst;
        end
        else
        begin
          ValRegister^ := VMemoryList.Items[Pred(VMemoryList.Count)];

          if (ValRegister^ = VMemoryList.Items[Pred(VMemoryList.Count)]) then
            Result := VMemoryList.DeleteLast;
        end;
      end;
    end;
  end;
end;

function ABCWriteMem(const AValueA : AnsiChar;
                     const AValueB : AnsiChar) : Boolean;
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  Result := false;

  if (VMemoryList <> nil) then
  begin
    ValRegister := ABCGetRegister(AValueA);
    ValValue    := ABCValue(AValueB);

    if (ValRegister <> nil) and
       (ValValue <> CEmpty) then
    begin
      if (ValValue = C0Val) then
        Result := (VMemoryList.InsertAsFirst(ValRegister^) and
                   (VMemoryList.Items[000] = ValRegister^))
      else
        Result := (VMemoryList.InsertAsLast(ValRegister^) and
                   (VMemoryList.Items[Pred(VMemoryList.Count)] = ValRegister^));
    end;
  end;
end;

procedure ABCSet(const AValueA : AnsiChar;
                 const AValueB : AnsiChar);
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  ValRegister := ABCGetRegister(AValueA);
  ValValue    := ABCValue(AValueB);

  if (ValRegister <> nil) and
     (ValValue <> CEmpty) then
    ValRegister^ := ValValue;
end;

procedure ABCSetNot(const AValueA : AnsiChar;
                    const AValueB : AnsiChar);
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  ValRegister := CatchRegister(AValueA);
  ValValue    := ABCValue(AValueB);

  if (ValRegister <> nil) and
     (ValValue <> CEmpty) then
    ValRegister^ := ValueNot(ValValue);
end;

function ABCTouchMemFirst(const AValueA : AnsiChar;
                          const AValueB : AnsiChar) : Boolean;
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  Result := false;

  if (VMemoryList <> nil) then
  begin
    if (VMemoryList.Count > 000) then
    begin
      ValRegister := ABCGetRegister(AValueA);
      ValValue    := ABCValue(AValueB);

      if (ValRegister <> nil) and
         (ValValue <> CEmpty) then
      begin
        ValRegister^ := VMemoryList.Items[000];

        if (ValRegister^ = VMemoryList.Items[000]) then
        begin
          if (ValValue = C1Val) then
            Result := VMemoryList.PutFirstToLast;
        end;
      end;
    end;
  end;
end;

function ABCTouchMemLast(const AValueA : AnsiChar;
                         const AValueB : AnsiChar) : Boolean;
var
  ValRegister : PAnsiChar;
  ValValue    : AnsiChar;
begin
  Result := false;

  if (VMemoryList <> nil) then
  begin
    if (VMemoryList.Count > 000) then
    begin
      ValRegister := ABCGetRegister(AValueA);
      ValValue    := ABCValue(AValueB);

      if (ValRegister <> nil) and
         (ValValue <> CEmpty) then
      begin
        ValRegister^ := VMemoryList.Items[Pred(VMemoryList.Count)];

        if (ValRegister^ = VMemoryList.Items[Pred(VMemoryList.Count)]) then
        begin
          if (ValValue = C0Val) then
            Result := VMemoryList.PutLastToFirst;
        end;
      end;
    end;
  end;
end;

function ABCValue(const AValue : AnsiChar) : AnsiChar;
begin
  Result := CEmpty;

  if IsRegister(AValue) then
    Result := ABCGetRegister(AValue)^
  else
  begin
    if IsValue(AValue) then
      Result := AValue
    else
    begin
      if IsExValue(AValue) then
        Result := ABCRandom;
    end;
  end;
end;

end.
