unit MainF;

interface

uses
  Windows,
  SysUtils,
  StdCtrls,
  Forms,
  Controls,
  Classes,
  ABCintrU,
  ABCsysU;

type
  TMainForm = class(TForm)
    AddAsciiCheckBox : TCheckBox;
    ErrorCheckBox    : TCheckBox;
    InputLabel       : TLabel;
    InputMemo        : TMemo;
    InterpretButton  : TButton;
    MessagesLabel    : TLabel;
    MessagesMemo     : TMemo;
    OutputLabel      : TLabel;
    OutputMemo       : TMemo;
    SuccessCheckBox  : TCheckBox;

    procedure FormCreate(Sender : TObject);
    procedure InterpretButtonClick(Sender : TObject);
  private
    { Private-Deklarationen }
    FReadFileCount   : LongInt;
    FReadFileHandle  : THandle;
    FReadFileSize    : LongInt;
    FWriteFileHandle : THandle;
  public
    { Public-Deklarationen }
    procedure CloseReadFile;
    procedure CloseWriteFile;
    procedure OpenReadFile(var ADone : Boolean);
    procedure OpenWriteFile(var ADone : Boolean);
    procedure ReadChar(var AChar : AnsiChar;
                       var ADone : Boolean);
    procedure WriteChar(const AChar : AnsiChar;
                        var ADone : Boolean);
  end;

var
  MainForm : TMainForm;

procedure ReadFile(var AValue : AnsiChar;
                   var ADone : Boolean);
procedure ReadScreen(var AValue : AnsiChar;
                     var ADone : Boolean);
procedure WriteFile(const AValue : AnsiChar;
                    var ADone : Boolean);
procedure WriteScreen(const AValue : AnsiChar;
                      var ADone : Boolean);

implementation

uses
  InputF;

{$R *.dfm}

procedure ReadFile(var AValue : AnsiChar;
                   var ADone : Boolean);
begin
  MainForm.ReadChar(AValue,
                    ADone);
end;

procedure ReadScreen(var AValue : AnsiChar;
                     var ADone : Boolean);
begin
  InputForm.ShowModal;

  if InputForm.FDone then
    AValue := AnsiChar(InputForm.InputSpinEdit.Value)
  else
    AValue := #000;

  ADone := InputForm.FDone;
end;

procedure WriteFile(const AValue : AnsiChar;
                    var ADone : Boolean);
begin
  MainForm.WriteChar(AValue,
                     ADone);
end;

procedure WriteScreen(const AValue : AnsiChar;
                      var ADone : Boolean);
begin
  if MainForm.AddAsciiCheckBox.Checked then
  begin
    if (AValue = #000) then
      MainForm.OutputMemo.Lines[Pred(MainForm.OutputMemo.Lines.Count)] := MainForm.OutputMemo.Lines[Pred(MainForm.OutputMemo.Lines.Count)] + #032 + #032#040 + IntToStr(Ord(AValue)) + #041
    else
      MainForm.OutputMemo.Lines[Pred(MainForm.OutputMemo.Lines.Count)] := MainForm.OutputMemo.Lines[Pred(MainForm.OutputMemo.Lines.Count)] + AValue + #032#040 + IntToStr(Ord(AValue)) + #041;

    MainForm.OutputMemo.Lines.Add('');
  end
  else
  begin
    if (AValue = #000) then
      MainForm.OutputMemo.Lines[Pred(MainForm.OutputMemo.Lines.Count)] := MainForm.OutputMemo.Lines[Pred(MainForm.OutputMemo.Lines.Count)] + #032
    else
      MainForm.OutputMemo.Lines[Pred(MainForm.OutputMemo.Lines.Count)] := MainForm.OutputMemo.Lines[Pred(MainForm.OutputMemo.Lines.Count)] + AValue;
  end;

  ADone := true;
end;

procedure TMainForm.FormCreate(Sender : TObject);
begin
  FReadFileCount   := 000;
  FReadFileHandle  := Invalid_Handle_Value;
  FReadFileSize    := 000;
  FWriteFileHandle := Invalid_Handle_Value;

  AddAsciiCheckBox.Checked := false;
  ErrorCheckBox.Checked    := true;
  SuccessCheckBox.Checked  := true;
end;

procedure TMainForm.InterpretButtonClick(Sender : TObject);
var
  Index       : LongInt;
  InterResult : AnsiString;
begin
  InitializeInterpreter(ReadFile,
                        ReadScreen,
                        WriteFile,
                        WriteScreen);

  MessagesMemo.Lines.Clear;
  OutputMemo.Lines.Clear;
//  OutputMemo.Lines.Add('');
  OutputMemo.Lines.Add(#013); // Memo-Bugfix (first memo-line has to contain a single #013)

  for Index := 000 to Pred(InputMemo.Lines.Count) do
  begin
    InterResult := InterpretLine(InputMemo.Lines[Index]);

    if (ErrorCheckBox.Checked or
        SuccessCheckBox.Checked) then
    begin
      if (((InterResult <> CError00Text) and
           ErrorCheckBox.Checked) or
          ((InterResult = CError00Text) and
           SuccessCheckBox.Checked)) then
        MessagesMemo.Lines.Add('Line' + #032 + IntToStr(Succ(Index)) + #058#032 + InterResult)
    end;
  end;

  CloseReadFile;
  CloseWriteFile;

  DeInitializeInterpreter;
end;

procedure TMainForm.CloseReadFile;
begin
  if (FReadFileHandle <> Invalid_Handle_Value) then
  begin
    FileClose(FReadFileHandle);

    FReadFileHandle := Invalid_Handle_Value;
  end;
end;

procedure TMainForm.CloseWriteFile;
begin
  if (FWriteFileHandle <> Invalid_Handle_Value) then
  begin
    FileClose(FWriteFileHandle);

    FWriteFileHandle := Invalid_Handle_Value;
  end;
end;

procedure TMainForm.OpenReadFile(var ADone : Boolean);
begin
  ADone := false;

  if (FReadFileHandle = Invalid_Handle_Value) then
  begin
    FReadFileHandle := FileOpen(ChangeFileExt(Application.ExeName,
                                              '.old'),
                                fmOpenRead or fmShareDenyNone);
    try
      if (FReadFileHandle <> Invalid_Handle_Value) then
      begin
        FReadFileCount := 000;
        FReadFileSize  := FileSeek(FReadFileHandle,
                                   000,
                                   002);
        FileSeek(FReadFileHandle,
                 000,
                 000);

        ADone := (FReadFileSize > 000);
      end;
    finally
      if not(ADone) then
        CloseReadFile;
    end;
  end;
end;

procedure TMainForm.OpenWriteFile(var ADone : Boolean);
var
  Index : LongWord;
begin
  ADone := false;

  if (FWriteFileHandle = Invalid_Handle_Value) then
  begin
    Index := 001;
    while FileExists(ChangeFileExt(Application.ExeName,
                                   '.new.' + IntToStr(Index))) do
      Inc(Index);

    FWriteFileHandle := FileCreate(ChangeFileExt(Application.ExeName,
                                               '.new.' + IntToStr(Index)));
    try
      ADone := (FWriteFileHandle <> Invalid_Handle_Value);
    finally
      if not(ADone) then
        CloseWriteFile;
    end;
  end;
end;

procedure TMainForm.ReadChar(var AChar : AnsiChar; var ADone : Boolean);
begin
  ADone := (FReadFileHandle <> Invalid_Handle_Value);
  if not(ADone) then
    OpenReadFile(ADone);

  if ADone then
  begin
    ADone := (FReadFileCount < FReadFileSize);

    if ADone then
    begin
      ADone := (FileRead(FReadFileHandle,
                         AChar,
                         1) = 1);

      Inc(FReadFileCount);
    end;
  end;
end;

procedure TMainForm.WriteChar(const AChar : AnsiChar; var ADone : Boolean);
begin
  ADone := (FWriteFileHandle <> Invalid_Handle_Value);
  if not(ADone) then
    OpenWriteFile(ADone);

  if ADone then
    ADone := (FileWrite(FWriteFileHandle,
                        AChar,
                        1) = 1);
end;

end.
